@@ -51,12 +51,13 @@ sub _draw {
     Imager::i_tt_cp($self->{id},$input{image}{IMG},
 		    $input{'x'}, $input{'y'}, $input{channel}, $input{size},
 		    $input{string}, length($input{string}),$input{aa},
-                    $input{utf8}); 
+                    $input{utf8}, $input{align}); 
   } else {
     Imager::i_tt_text($self->{id}, $input{image}{IMG}, 
 		      $input{'x'}, $input{'y'}, $input{color},
 		      $input{size}, $input{string}, 
-		      length($input{string}), $input{aa}, $input{utf8}); 
+		      length($input{string}), $input{aa}, $input{utf8},
+                      $input{align}); 
   }
 }
 
@@ -92,7 +93,7 @@ sub glyph_names {
 
   my $string = $input{string};
   defined $string
-    or return Imager->_seterror("no string parameter passed to glyph_names");
+    or return Imager->_set_error("no string parameter passed to glyph_names");
   my $utf8 = _first($input{utf8} || 0);
 
   Imager::i_tt_glyph_name($self->{id}, $string, $utf8);
